package com.example.demo;
import com.example.demo.mq.LogEvent;
import com.example.demo.mq.LogEventFactory;
import com.example.demo.mq.LogEventHandler;
import com.lmax.disruptor.BlockingWaitStrategy;
import com.lmax.disruptor.RingBuffer;
import com.lmax.disruptor.dsl.Disruptor;
import com.lmax.disruptor.dsl.ProducerType;
import org.junit.jupiter.api.Test;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.atomic.AtomicInteger;
public class DisruptorTest {
    @Test
    public void test1() {
        int bufferSize = 1024;
        // 获取 Disruptor 对象
        // Disruptor<LogEvent> disruptor = new Disruptor<>(LogEvent::new, bufferSize, DaemonThreadFactory.INSTANCE);
        Disruptor<LogEvent> disruptor = getLogEventDisruptor();
        // 绑定处理事件的Handler对象
        // 以设置多个处理事件的 Handler，并且可以灵活的设置消费者的处理顺序，串行，并行都是可以的。
        disruptor.handleEventsWith(new LogEventHandler());
        // 下面的代码表示 Handler1 和 Handler2 是并行执行，最后再执行 Handler3
        // disruptor.handleEventsWith(new Handler1(), new Handler2()).handleEventsWith(new Handler3());
        // 启动 Disruptor
        disruptor.start();
        // 获取保存事件的环形数组（RingBuffer）
        RingBuffer<LogEvent> ringBuffer = disruptor.getRingBuffer();
        // 发布 5个 个事件
        for (int i = 1; i <= 5; i++) {
            try {
                // 初始化 Event，对其赋值
                ringBuffer.publishEvent((event, sequence, buffer) -> event.setMessage("这是日志消息 " + System.nanoTime()));
            }catch (Exception e){
                e.printStackTrace();
            }
        }
        // 关闭 Disruptor
        disruptor.shutdown();
        // 模拟堵塞主线程不退出
        try {
            while (true) {
                Thread.sleep(5000);
            }
        } catch (Exception e) {

        }
    }
    private static Disruptor<LogEvent> getLogEventDisruptor() {
        // 创建 LogEvent 的工厂
        LogEventFactory logEventFactory = new LogEventFactory();
        // Disruptor 的 RingBuffer 缓存大小，环形缓冲器的大小，必须是2的幂
        int bufferSize = 1024;
        // 生产者的线程工厂
        ThreadFactory threadFactory = new ThreadFactory() {
            // 线程安全的计数类
            // https://www.javacui.com/Theory/691.html
            final AtomicInteger threadNum = new AtomicInteger(0);
            @Override
            public Thread newThread(Runnable r) {
                System.out.println("LogEventThread" + " [#" + threadNum.incrementAndGet() + "] 已创建");
                return new Thread(r, "LogEventThread" + " [#" + threadNum.incrementAndGet() + "]");
            }
        };
        //实例化 Disruptor
        return new Disruptor<>(
                logEventFactory, // 自定义的事件工厂
                bufferSize,      // 自定义的时间工厂
                threadFactory,   // 自定义的线程工厂。Disruptor的默认线程池是自定义的，我们只需要传入线程工厂即可。
                ProducerType.MULTI,          // 指定是单个事件发布者模式还是多个事件发布者模式
                new BlockingWaitStrategy()); // 阻塞等待策略
    }
}